package javax.management;

import com.sun.jmx.mbeanserver.Util;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.SortedMap;
import java.util.Comparator;
import java.io.InvalidObjectException;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map;

public class ImmutableDescriptor implements Descriptor
{
    private static final long serialVersionUID = 8853308591080540165L;
    private final String[] names;
    private final Object[] values;
    private transient int hashCode;
    public static final ImmutableDescriptor EMPTY_DESCRIPTOR;
    
    public ImmutableDescriptor(String[] array, Object[] array2) {
        this(makeMap(array, array2));
    }
    
    public ImmutableDescriptor(String... array) {
        this(makeMap(array));
    }
    
    public ImmutableDescriptor(Map<String, ?> map) {
        this.hashCode = -1;
        if (map == null) {
            throw new IllegalArgumentException("Null Map");
        }
        TreeMap treeMap = new TreeMap(String.CASE_INSENSITIVE_ORDER);
        for (Map.Entry entry : map.entrySet()) {
            String str = (String)entry.getKey();
            if (str == null || str.equals("")) {
                throw new IllegalArgumentException("Empty or null field name");
            }
            if (treeMap.containsKey(str)) {
                throw new IllegalArgumentException("Duplicate name: " + str);
            }
            treeMap.put(str, entry.getValue());
        }
        int size = treeMap.size();
        this.names = (String[])treeMap.keySet().toArray(new String[size]);
        this.values = treeMap.values().toArray(new Object[size]);
        return;
    }
    
    private Object readResolve() throws InvalidObjectException {
        int n = 0;
        if (this.names == null || this.values == null || this.names.length != this.values.length) {
            n = 1;
        }
        if (n == 0) {
            if (this.names.length == 0 && this.getClass() == ImmutableDescriptor.class) {
                return ImmutableDescriptor.EMPTY_DESCRIPTOR;
            }
            Comparator<String> case_INSENSITIVE_ORDER = String.CASE_INSENSITIVE_ORDER;
            String s = "";
            int i = 0;
            while (i < this.names.length) {
                if (this.names[i] == null || case_INSENSITIVE_ORDER.compare(s, this.names[i]) >= 0) {
                    n = 1;
                    break;
                }
                s = this.names[i];
                ++i;
            }
        }
        if (n != 0) {
            throw new InvalidObjectException("Bad names or values");
        }
        return this;
    }
    
    private static SortedMap<String, ?> makeMap(String[] array, Object[] array2) {
        if (array == null || array2 == null) {
            throw new IllegalArgumentException("Null array parameter");
        }
        if (array.length != array2.length) {
            throw new IllegalArgumentException("Different size arrays");
        }
        TreeMap treeMap = new TreeMap(String.CASE_INSENSITIVE_ORDER);
        int i = 0;
        while (i < array.length) {
            String str = array[i];
            if (str == null || str.equals("")) {
                throw new IllegalArgumentException("Empty or null field name");
            }
            if (treeMap.put(str, array2[i]) != null) {
                throw new IllegalArgumentException("Duplicate field name: " + str);
            }
            ++i;
        }
        return treeMap;
    }
    
    private static SortedMap<String, ?> makeMap(String[] array) {
        if (array == null) {
            throw new IllegalArgumentException("Null fields parameter");
        }
        String[] array2 = new String[array.length];
        String[] array3 = new String[array.length];
        int i = 0;
        while (i < array.length) {
            String str = array[i];
            int index = str.indexOf(61);
            if (index < 0) {
                throw new IllegalArgumentException("Missing = character: " + str);
            }
            array2[i] = str.substring(0, index);
            array3[i] = str.substring(index + 1);
            ++i;
        }
        return makeMap(array2, array3);
    }
    
    public static ImmutableDescriptor union(Descriptor... array) {
        int nonEmpty = findNonEmpty(array, 0);
        if (nonEmpty < 0) {
            return ImmutableDescriptor.EMPTY_DESCRIPTOR;
        }
        if (array[nonEmpty] instanceof ImmutableDescriptor && findNonEmpty(array, nonEmpty + 1) < 0) {
            return (ImmutableDescriptor)array[nonEmpty];
        }
        TreeMap treeMap = new TreeMap(String.CASE_INSENSITIVE_ORDER);
        ImmutableDescriptor empty_DESCRIPTOR = ImmutableDescriptor.EMPTY_DESCRIPTOR;
        for (Descriptor descriptor : array) {
            if (descriptor != null) {
                String[] array2;
                if ((descriptor instanceof ImmutableDescriptor)) {
                    ImmutableDescriptor immutableDescriptor = (ImmutableDescriptor)descriptor;
                    array2 = immutableDescriptor.names;
                    if (immutableDescriptor.getClass() == ImmutableDescriptor.class && array2.length > empty_DESCRIPTOR.names.length) {
                        empty_DESCRIPTOR = immutableDescriptor;
                    }
                }
                else {
                    array2 = descriptor.getFieldNames();
                }
                for (String str : array2) {
                    Object fieldValue = descriptor.getFieldValue(str);
                    Object put = treeMap.put(str, fieldValue);
                    if (put != null) {
                        int n;
                        if (put.getClass().isArray()) {
                            n = Arrays.deepEquals(new Object[] { put }, new Object[] { fieldValue });
                        }
                        else {
                            n = put.equals(fieldValue);
                        }
                        if (n == 0) {
                            throw new IllegalArgumentException("Inconsistent values for descriptor field " + str + ": " + put + " :: " + fieldValue);
                        }
                    }
                }
            }
        }
        if (empty_DESCRIPTOR.names.length == treeMap.size()) {
            return empty_DESCRIPTOR;
        }
        return new ImmutableDescriptor(treeMap);
    }
    
    private static boolean isEmpty(Descriptor descriptor) {
        if (descriptor == null) {
            return true;
        }
        if ((descriptor instanceof ImmutableDescriptor)) {
            return ((ImmutableDescriptor)descriptor).names.length == 0;
        }
        return descriptor.getFieldNames().length == 0;
    }
    
    private static int findNonEmpty(Descriptor[] array, int i) {
        while (i < array.length) {
            if (!isEmpty(array[i])) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    private int fieldIndex(String key) {
        return Arrays.binarySearch(this.names, key, String.CASE_INSENSITIVE_ORDER);
    }
    
    @Override
    public final Object getFieldValue(String s) {
        checkIllegalFieldName(s);
        int fieldIndex = this.fieldIndex(s);
        if (fieldIndex < 0) {
            return null;
        }
        Object o = this.values[fieldIndex];
        if (o == null || !o.getClass().isArray()) {
            return o;
        }
        if ((o instanceof Object[])) {
            return ((Object[])(Object[])o).clone();
        }
        int length = Array.getLength(o);
        Object instance = Array.newInstance(o.getClass().getComponentType(), length);
        System.arraycopy(o, 0, instance, 0, length);
        return instance;
    }
    
    @Override
    public final String[] getFields() {
        String[] array = new String[this.names.length];
        Object string;
        for (int i = 0; i < array.length; array[i] = this.names[i] + "=" + string, ++i) {
            string = this.values[i];
            if (string == null) {
                string = "";
            }
            else if (!(string instanceof String)) {
                string = "(" + string + ")";
            }
        }
        return array;
    }
    
    @Override
    public final Object[] getFieldValues(String... array) {
        if (array == null) {
            return (Object[])this.values.clone();
        }
        Object[] array2 = new Object[array.length];
        for (int i = 0; i < array.length; ++i) {
            String s = array[i];
            if (s != null && !s.equals("")) {
                array2[i] = this.getFieldValue(s);
            }
        }
        return array2;
    }
    
    @Override
    public final String[] getFieldNames() {
        return (String[])this.names.clone();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof Descriptor)) {
            return false;
        }
        String[] a;
        if ((o instanceof ImmutableDescriptor)) {
            a = ((ImmutableDescriptor)o).names;
        }
        else {
            a = ((Descriptor)o).getFieldNames();
            Arrays.sort(a, String.CASE_INSENSITIVE_ORDER);
        }
        if (this.names.length != a.length) {
            return false;
        }
        int i = 0;
        while (i < this.names.length) {
            if (!this.names[i].equalsIgnoreCase(a[i])) {
                return false;
            }
            ++i;
        }
        Object[] a2;
        if ((o instanceof ImmutableDescriptor)) {
            a2 = ((ImmutableDescriptor)o).values;
        }
        else {
            a2 = ((Descriptor)o).getFieldValues(a);
        }
        return Arrays.deepEquals(this.values, a2);
    }
    
    @Override
    public int hashCode() {
        if (this.hashCode == -1) {
            this.hashCode = Util.hashCode(this.names, this.values);
        }
        return this.hashCode;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("{");
        for (int i = 0; i < this.names.length; ++i) {
            if (i > 0) {
                sb.append(", ");
            }
            sb.append(this.names[i]).append("=");
            Object substring = this.values[i];
            if (substring != null && substring.getClass().isArray()) {
                String deepToString = Arrays.deepToString(new Object[] { substring });
                substring = deepToString.substring(1, deepToString.length() - 1);
            }
            sb.append(String.valueOf(substring));
        }
        return sb.append("}").toString();
    }
    
    @Override
    public boolean isValid() {
        return true;
    }
    
    @Override
    public Descriptor clone() {
        return this;
    }
    
    @Override
    public final void setFields(String[] array, Object[] array2) throws RuntimeOperationsException {
        if (array == null || array2 == null) {
            illegal("Null argument");
        }
        if (array.length != array2.length) {
            illegal("Different array sizes");
        }
        for (int i = 0; i < array.length; ++i) {
            checkIllegalFieldName(array[i]);
        }
        for (int j = 0; j < array.length; ++j) {
            this.setField(array[j], array2[j]);
        }
    }
    
    @Override
    public final void setField(String s, Object obj) throws RuntimeOperationsException {
        checkIllegalFieldName(s);
        int fieldIndex = this.fieldIndex(s);
        if (fieldIndex < 0) {
            unsupported();
        }
        Object o = this.values[fieldIndex];
        if (!((o != null) ? o.equals(obj) : (obj == null))) {
            unsupported();
        }
    }
    
    @Override
    public final void removeField(String s) {
        if (s != null && this.fieldIndex(s) >= 0) {
            unsupported();
        }
    }
    
    static Descriptor nonNullDescriptor(Descriptor descriptor) {
        if (descriptor == null) {
            return ImmutableDescriptor.EMPTY_DESCRIPTOR;
        }
        return descriptor;
    }
    
    private static void checkIllegalFieldName(String s) {
        if (s == null || s.equals("")) {
            illegal("Null or empty field name");
        }
    }
    
    private static void unsupported() {
        throw new RuntimeOperationsException(new UnsupportedOperationException("Descriptor is read-only"));
    }
    
    private static void illegal(String s) {
        throw new RuntimeOperationsException(new IllegalArgumentException(s));
    }
    
    static {
        EMPTY_DESCRIPTOR = new ImmutableDescriptor(new String[0]);
    }
}
